import jQuery from "jquery";

const SAVE_BUTTON_ID = "div#ks-editor #toolbar-save";

export default class ManagedData {
    constructor(data) {
        this.data = data;
        this.history = []; // Newest state is always at 0
        this.historyPosition = 0;
        this.savedHistoryId = 0;

        this.storeCurrentData("Initial state", false);
    }

    updateUnsavedChangesHandler() {
        if (this.hasUnsavedChanges()) {
            jQuery(SAVE_BUTTON_ID).removeClass("hidden");
            window.addEventListener("beforeunload", this.handleBeforeUnload);
        } else {
            jQuery(SAVE_BUTTON_ID).addClass("hidden");
            window.removeEventListener("beforeunload", this.handleBeforeUnload);
        }
    }

    handleBeforeUnload(e) {
        var confirmationMessage =
            "If you leave before saving, unsaved changes will be lost.";

        (e || window.event).returnValue = confirmationMessage; //Gecko + IE
        return confirmationMessage; //Gecko + Webkit, Safari, Chrome etc.
    }

    hasUnsavedChanges() {
        return this.history[this.historyPosition].id !== this.savedHistoryId;
    }

    saveChanges() {
        this.savedHistoryId = this.history[this.historyPosition].id;
        this.updateUnsavedChangesHandler();
    }

    onUndo() {}
    onRedo() {}

    undo() {
        if (this.step(1)) {
            this.updateUnsavedChangesHandler();
            this.onUndo();
            return true;
        } else {
            return false;
        }
    }

    redo() {
        if (this.step(-1)) {
            this.updateUnsavedChangesHandler();
            this.onRedo();
            return true;
        } else {
            return false;
        }
    }

    step(direction = 1) {
        var newHistoryPosition = this.historyPosition + Math.sign(direction);

        if (
            newHistoryPosition >= this.history.length ||
            newHistoryPosition < 0
        ) {
            return false;
        }

        this.historyPosition = newHistoryPosition;
        this.data = JSON.parse(this.history[this.historyPosition].data);

        return true;
    }

    storableData(data) {
        return data;
    }

    storeCurrentData(description, relevantChanges = true) {
        var formattedData = this.storableData(this.data);

        var nextId = 0;
        if (this.history.length > 0) {
            nextId = this.history[0].id;

            // Keep same as previous id, if nothing relevant changed
            // Otherwise, increase by one
            if (relevantChanges) {
                nextId++;
            }
        }

        // Forget about the currently stored potential future
        this.history.splice(0, this.historyPosition);
        this.historyPosition = 0;

        this.history.unshift({
            description: description,
            data: JSON.stringify(formattedData), // Creating a deep copy
            id: nextId,
        });

        this.updateUnsavedChangesHandler();
    }
}
